;;; Directory Local Variables
;;; For more information see (info "(emacs) Directory Variables")

((nil
  (bug-reference-url-format . "https://github.com/ledger/ledger-mode/issues/%s")
  (bug-reference-bug-regexp . "\\(GH-?\\|[Rr]equest ?#?\\|[Bb]ug ?#?\\|[Pp]atch ?#\\|RFE ?#\\|PR [a-z-+]+/\\)\\([0-9]+\\(?:#[0-9]+\\)?\\)")
  (indent-tabs-mode . nil)
  (fill-column . 80)))
